<?php
/**
 * @see Alinks_Menu_Abstract
 */
require_once('Alinks/Menu/Abstract.php');

/**
 * @see Alinks_Form
 */
require_once('Alinks/Form.php');

/**
 * Displays the settings menu
 *
 * Copyright (C) 2008  Sean Hickey
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @author Sean Hickey <sean@headzoo.com>
 * @version $Id: Settings.php 382 2008-07-03 05:31:10Z sean $
 * @package Alinks_Menu
 */
class Alinks_Menu_Settings extends Alinks_Menu_Abstract
{
	/**
	 * The settings forms for the factories
	 *
	 * @var Alinks_Form[]
	 */
	protected static $_forms = null;
	
	/**
	 * The menu to show to the user
	 *
	 * @var string
	 */
	protected static $_show = 'AlinksMenus/Settings.php';
	
	/**
	 * Inits the menu
	 *
	 * Sets up CSS/JavaScript, and processes any submitted forms.
	 */
	public static function init()
	{
		if (isset($_POST[Alinks_Form::FORM_ALINKS]) && $_POST[Alinks_Form::FORM_ALINKS] != 'uninstall') {
			self::processForm();
		} else if (isset($_POST[Alinks_Form::FORM_ALINKS]) && $_POST[Alinks_Form::FORM_ALINKS] == 'uninstall') {
			self::_uninstall();
		}
	}
	
	/**
	 * Shows the settings menu
	 */
	public static function show()
	{
		if (null === self::$_forms) {
			self::_setupSettingsForms();
		}
		include(self::$_show);
	}
	
	/**
	 * Sets up the settings form for all factories
	 *
	 * Gets the settings forms from all the factories.
	 */
	protected static function _setupSettingsForms()
	{
		$factories = Alinks_Registry::get('Alinks_Factory_Keeper')->fetchAll();
		foreach($factories as $factory) {
			if (!$factory->hasSettingsForm()) {
				continue;
			}
			$form = $factory->getSettingsForm();
			if (!$form instanceof Alinks_Form) {
				continue;
			}
			$settings = Alinks_Settings::get($factory->getSafeName(), null, array());
			$form->setValues($settings);
			self::$_forms[$factory->getSafeName()] = $form;
		}
	}
	
	/**
	 * Processes the submitted settings form
	 */
	public static function processForm()
	{
		check_admin_referer(Alinks_Form::NONCE_ACTION);
		self::_setupSettingsForms();
		if (!isset($_POST[Alinks_Form::FORM_FACTORY])) {
			return;
		}
		
		$factory = $_POST[Alinks_Form::FORM_FACTORY];
		$_POST = self::_getFilteredForm();
		if ('alinks' != $factory) {
			self::$_forms[$factory]->setValues($_POST);
			if (self::$_forms[$factory]->hasInvalid()) {
				return;
			}
		}

		foreach($_POST as $key => $value) {
			if (Alinks_Settings::set($factory, $key, $value)) {
				/**
				 * @see Alinks_Post_Parser
				 */
				require_once('Alinks/Post/Parser.php');
				$cacher = Alinks_Registry::get('cacher');
				if ($cacher) {
					$cacher->flush(Alinks_Post_Parser::CACHE_NS_COMMENTS);
					$cacher->flush(Alinks_Post_Parser::CACHE_NS_POSTS);
				}
			}
		}
		
		add_action('admin_notices', array('Alinks_Menu_Settings', 'adminNotices'));
		self::$_success_msg = 'Your settings have been updated!';
		self::$_success_title = 'Settings Updated';
	}
	
	/**
	 * Uninstalls aLinks!
	 */
	protected static function _uninstall()
	{
		check_admin_referer(Alinks_Form::NONCE_ACTION . '_uninstall');
		if (!isset($_POST['alinks_uninstall_step'])) {
			return;
		}
		if ($_POST['alinks_uninstall_step'] == 1) {
			self::$_show = 'AlinksMenus/Uninstall.php';
			return;
		}
		if ($_POST['alinks_uninstall_step'] == 2 && $_POST['action'] == 'Cancel') {
			return;
		}
		
		/**
		 * @see Alinks_Uninstall
		 */
		require_once('Alinks/Uninstall.php');
		Alinks_Uninstall::uninstall();
	}
	
	/**
	 * Filters a submitted form
	 *
	 * Removes elements from the submitted form values that aren't needed,
	 * and deals with checkbox values.
	 *
	 * @return array
	 */
	protected static function _getFilteredForm()
	{
		$values = array();
		foreach($_POST as $key => $value) {
			/**
			 * Checkbox values aren't submitted when they're not checked. So we put
			 * the unchecked value in a hidden field. The hidden field name is the checkbox
			 * field name, prefixed with a double underscore. Using that information, we
			 * can determine if a checkbox is unchecked.
			 */
			if (substr($key, 0, 2) == '__') {
				$checkbox_key = substr($key, 2);
				if (!isset($_POST[$checkbox_key])) {
					$key = $checkbox_key;
					$value = 0;
				} else {
					continue;
				}
			/**
			 * Hidden elements that provide instructions to aLinks are prefixed with
			 * a single underscore. We can ignore these.
			 */
			} else if (substr($key, 0, 1) == '_') {
				continue;
			}
			
			/**
			 * WordPress strips slashes from POST data if magic quotes is
			 * turned on, and then adds them again right afterwards. We escape
			 * our own data before DB inserting, so we will get rid of those
			 * slashes.
			 */
			$values[$key] = stripslashes($value);
		}
		
		return $values;
	}
}
