<?php
/**
 * Used to get and set aLinks and factory settings
 *
 * The "gateway" to aLinks and factory settings. All interaction
 * between aLinks and the settings table in the database go through
 * this class.
 *
 * Copyright (C) 2008  Sean Hickey
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @author Sean Hickey <sean@headzoo.com>
 * @version $Id: Settings.php 374 2008-07-03 04:50:34Z sean $
 * @package Alinks_Settings
 */
class Alinks_Settings
{
	/**
	 * Max number of keyphrases to parse in each post or comment
	 */
	const MAX_LINKS = 'maxlinks';
	const DEFAULT_MAX_LINKS = 1;
	
	/**
	 * Max number of links to have in each post
	 */
	const MAX_LINKS_POST = 'maxlinks_post';
	const DEFAULT_MAX_LINKS_POST = 10;
	
	/**
	 * Max number of links per page
	 */
	const MAX_LINKS_PAGE = 'maxlinks_page';
	const DEFAULT_MAX_LINKS_PAGE = -1;
	
	/**
	 * Whether or not to parse the comments too
	 */
	const PARSE_COMMENTS = 'parse_comments';
	const DEFAULT_PARSE_COMMENTS = 0;
	
	/**
	 * HTML to add inside each anchor tag
	 */
	const INNER_HTML = 'inner_html';
	const DEFAULT_INNER_HTML = '';
	
	/**
	 * Classes to add to all created links
	 */
	const CLASSES = 'classes';
	const DEFAULT_CLASSES = 'alinks_link';
	
	/**
	 * To open links in a new window
	 */
	const NEW_WINDOW = 'new_window';
	const DEFAULT_NEW_WINDOW = 0;
	
	/**
	 * The default export document type
	 */
	const DEFAULT_EXPORT = 'default_export';
	const DEFAULT_DEFAULT_EXPORT = 'xml';
	
	/**
	 * The settings
	 *
	 * @var array
	 */
	protected static $_settings = null;
	
	/**
	 * Instance of wpdb
	 *
	 * @var wpdb
	 */
	protected static $_wpdb;
	
	/**
	 * Instance of Alinks_Cacher_Interface
	 *
	 * @var Alinks_Cacher_Interface
	 */
	protected static $_cacher = null;
	
	/**
	 * Options that WordPress has stored, not aLinks
	 *
	 * If the value for these setting keys is stored in the
	 * wp_options table, and not alinks_settings.
	 *
	 * @var array
	 */
	protected static $_pass = array(
		'home',
		'alinks_version',
		'autolinks_keywords',
		'autolinks_amazonkeywords'
	);
	
	/**
	 * Sets the database adapter to query the database
	 *
	 * @param wpdb $wpdb The wpdb adapter
	 * @param string $table_prefix The DB table prefix
	 * @return bool
	 */
	public static function setDb(wpdb $wpdb, $table_prefix)
	{
		$wpdb->alinks_settings = $table_prefix . 'alinks_settings';
		self::$_wpdb = $wpdb;
		return true;
	}
	
	/**
	 * Sets the cacher for caching data
	 *
	 * Sets an instance of Alinks_Cacher_Interface, so setting values
	 * can be get and set in a cache, and not the database.
	 *
	 * @param Alinks_Cacher_Interface $cacher The cacher
	 * @return bool
	 */
	public static function setCacher(Alinks_Cacher_Interface $cacher)
	{
		self::$_cacher = $cacher;
		return true;
	}
	
	/**
	 * Gets a stored setting
	 *
	 * Returns the value for the given setting. The value of $default
	 * will be returned if the value does not exist.
	 *
	 * All the values for the specified factory will be returned if $key
	 * is null. If $factory is null, then all the settings will be
	 * returned.
	 *
	 * @param string $factory The factory the setting belongs to
	 * @param string $key The option key
	 * @param mixed $default The default
	 * @return mixed
	 */
	public static function get($factory = null, $key = null, $default = null)
	{
		$value = $default;
		if (null === self::$_settings) {
			self::_loadSettings();
		}
		
		/**
		 * First check if this is an option stored in WordPress's own
		 * options system.
		 */
		if (in_array($key, self::$_pass)) {
			$result = get_option($key);
			/**
			 * I hate that WordPress's get_option() function returns false
			 * if the value isn't set. Maybe... just maybe the stored value
			 * is the boolean false? How are we to know that the value isn't
			 * stored?
			 */
			if (false !== $result) {
				$value = $result;
			}
		} else if (null === $factory) {
			$value = self::$_settings;
		} else if (null === $key) {
			if (isset(self::$_settings[$factory])) {
				$value = self::$_settings[$factory];
			}
		} else {
			if (isset(self::$_settings[$factory][$key])) {
				$value = self::$_settings[$factory][$key];
			}
		}
		
		return $value;
	}
	
	/**
	 * Sets an option
	 *
	 * Sets an option. Neither the key or value should be SQL
	 * escaped. The method will escape them.
	 *
	 * Returns true if a value was actually changed. False if not.
	 *
	 * @param string $factory The factory the setting belongs to
	 * @param string $key The option key
	 * @param string $value The option value
	 * @return bool
	 */
	public static function set($factory, $key, $value)
	{
		if (null === self::$_settings) {
			self::_loadSettings();
		}
		
		/**
		 * If the key points to one of WordPress's own options, then
		 * we set the option through WP.
		 */
		if (in_array($key, self::$_pass)) {
			return (bool)set_option($key, $value);
		}
		
		$needs_updating = false;
		if (!isset(self::$_settings[$factory])) {
			self::$_settings[$factory] = array();
			$needs_updating = true;
		}
		if (!isset(self::$_settings[$factory][$key])) {
			self::$_settings[$factory][$key] = null;
			$needs_updating = true;
		}
		
		/**
		 * We're not going to update the database if the new value
		 * is the same as the old value.
		 */
		if (self::$_settings[$factory][$key] != $value) {
			self::$_settings[$factory][$key] = $value;
			$needs_updating = true;
		}

		if ($needs_updating) {
			$result = self::_update($factory, $key, $value);
		} else {
			$result = false;
		}
		
		return $result;
	}
	
	/**
	 * Deletes a setting
	 *
	 * Completely removes a setting from the database. Returns true if
	 * the value was actually changed. False if not.
	 *
	 * If $key is null, then all of the settings for the specified
	 * factory will be removed.
	 *
	 * @param string $factory The factory the setting belongs to
	 * @param string $key The setting key
	 * @return bool
	 */
	public static function delete($factory, $key = null)
	{
		if (null === self::$_settings) {
			self::_loadSettings();
		}
		
		/**
		 * If the key points to one of WordPress's own options, then
		 * we delete the option through WP.
		 */
		if (in_array($key, self::$_pass)) {
			$results = (bool)delete_option($key);
		} else if (isset(self::$_settings[$factory])) {
			if (null === $key) {
				foreach(array_keys(self::$_settings[$factory]) as $key) {
					self::delete($factory, $key);
				}
			} else {
				unset(self::$_settings[$factory][$key]);
				$results = self::_update($factory, $key, null);
			}
		}
		
		return $results;
	}
	
	/**
	 * Updates a setting in the database
	 *
	 * Setting $value to null will delete the setting from the database. Returns
	 * if a value was actually changed.
	 *
	 * @param string $factory The factory the setting belongs to
	 * @param string $key The option key
	 * @param string $value The option value
	 * @return bool
	 */
	protected static function _update($factory, $key, $value)
	{
		$factory 	= self::$_wpdb->escape($factory);
		$key 		= self::$_wpdb->escape($key);
		
		if (null === $value) {
			$sql = 'DELETE FROM ' . self::$_wpdb->alinks_settings . " WHERE `factory` = '$factory' AND `key` = '$key' LIMIT 1";
		} else {
			$value = self::$_wpdb->escape($value);
			$sql = 'INSERT INTO ' . self::$_wpdb->alinks_settings .
			" (`id`, `factory`, `key`, `value`)
			VALUES(null, '$factory', '$key', '$value')
			ON DUPLICATE KEY UPDATE `value` = '$value'";
		}
		
		$results = self::$_wpdb->query($sql);
		if ($results) {
			if (null !== self::$_cacher) {
				self::$_cacher->flush('settings');
			}
		}
		
		return (bool)$results;
	}
	
	/**
	 * Loads all the settings from the database
	 */
	protected static function _loadSettings()
	{
		if (null !== self::$_cacher && self::$_cacher->exists('all', 'settings')) {
			self::$_settings = self::$_cacher->get('all', 'settings');
		} else {
			$sql = 'SELECT * FROM ' . self::$_wpdb->alinks_settings;
			$results = self::$_wpdb->get_results($sql, ARRAY_A);
			self::$_settings = array();
			
			foreach((array)$results as $result) {
				self::$_settings[$result['factory']][$result['key']] = $result['value'];
			}
			self::$_cacher->set('all', 'settings', self::$_settings);
		}
	}
}
